﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Memcached_Ketama
{
    public class KetamaNodeLocator
    {
        //原文中的JAVA类TreeMap实现了Comparator方法，这里我图省事，直接用了net下的SortedList，其中Comparer接口方法）
        private SortedList<long, string> ketamaNodes = new SortedList<long, string>();
        private long[] hostKeysArr = new long[] { };
        private HashAlgorithmSPY hashAlg;
        private int numReps = 250;

        //此处参数与JAVA版中有区别，因为使用的静态方法，所以不再传递HashAlgorithm alg参数
        public KetamaNodeLocator(List<string> nodes, int nodeCopies)
        {
            ketamaNodes = new SortedList<long, string>();

            numReps = nodeCopies;
            //对所有节点，生成nCopies个虚拟结点
            foreach (string node in nodes)
            {
                //每四个虚拟结点为一组
                for (int i = 0; i < numReps / 4; i++)
                {
                    //getKeyForNode方法为这组虚拟结点得到惟一名称 
                    byte[] digest = HashAlgorithmSPY.computeMd5(node + i);
                    /** Md5是一个16字节长度的数组，将16字节的数组每四个字节一组，分别对应一个虚拟结点，这就是为什么上面把虚拟结点四个划分一组的原因*/
                    for (int h = 0; h < 4; h++)
                    {
                        long m = HashAlgorithmSPY.hash(digest, h);
                        ketamaNodes[m] = node;
                    }
                }
            }

            hostKeysArr = ketamaNodes.Keys.OrderBy(x => x).ToArray();
        }

        public string GetPrimary(string k)
        {
            byte[] digest = HashAlgorithmSPY.computeMd5(k);
            string rv = GetNodeForKey(HashAlgorithmSPY.hash(digest, 0));
            return rv;
        }

        string GetNodeForKey(long hash)
        {
            string rv;
            long key = hash;
            //如果找到这个节点，直接取节点，返回   
            if (!ketamaNodes.ContainsKey(key))
            {
                //得到大于当前key的那个子Map，然后从中取出第一个key，就是大于且离它最近的那个key 说明详见: http://www.javaeye.com/topic/684087
                var tailMap = from coll in ketamaNodes
                              where coll.Key > hash
                              select new { coll.Key };
                if (tailMap == null || tailMap.Count() == 0)
                    key = ketamaNodes.FirstOrDefault().Key;
                else
                    key = tailMap.FirstOrDefault().Key;
            }
            rv = ketamaNodes[key];
            return rv;



            ////寻找与当前hash值相等的Server. 
            //int i = Array.BinarySearch(hostKeysArr, hash);

            ////如果i小于零则表示没有hash值相等的虚拟节点
            //if (i < 0)
            //{
            //    //将i继续按位求补,得到数组中第一个大于当前hash值的虚拟节点
            //    i = ~i;

            //    //如果按位求补后的i大于等于数组的大小,则表示数组中没有大于当前hash值的虚拟节点
            //    //此时直接取第一个server
            //    if (i >= hostKeysArr.Length)
            //    {
            //        i = 0;
            //    }
            //}

            ////根据虚拟节点的hash key 返回对应的真实server host地址
            //return ketamaNodes[hostKeysArr[i]];

        }
    }
}
